#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>


#define TRUE 1
#define FALSE 0
#define ERR -1



/* Find the frequency of a tone. Returns the double-precision result when
   given the tone number and frequency of A-440. (A5 frequency, normally
   440.00 Hz) */


double find_frequency(int semitone,double pitch)
{
double c;	/* Reference pitch for start of an octave, m3 above A4. Needed
               for downward dereferencing of octaves. */


c = 1.189207115 * pitch;

			/* Find power of 2 for number of octaves to span. Spanning the
			 octaves this way reduces rounding error that might result from
			 cumulative operation of POW(). */

c = c * pow( 2. , (double) (semitone / 12 - 4));

			/* Compute frequency inside THIS octave */

return( c * pow( 2. , (double) (semitone % 12) / 12. ) );
}




/* int find_partials( double f1, double f2, int *ndx_limit1, int *ndx_limit2,
		int upper_window, double stretch)

   This function examines the idealized harmonic overtone series of tones
   'f1' and 'f2' for audible beats. The upper limits for each partial are
   contained in *ndx_limit1/2, respectively. A value of 1 represents the
   fundamental or 'first partial.'

   The value "upper_window" specifies a termination value for the beat
   frequency; in other words, the FIRST beat frequency that is below this
   window will be reported.

   The actual beat frequency is not directly reported; instead, the
   parameters *ndx_limit1/2 are modified.

   STRETCH is the uniform inharmicity factor. 1.00 represents pure
   overtone series that exactly correspond with Fourier harmonic 
   series; i.e., there is NO inharmicity when STRETCH == 1.00.
   In this version of the function, STRETCH is ignored, but in later
   versions, it will be used to compute the modified overtone series
   typical of short vertical pianos.


   The function RETURNS <FALSE> if no "suitably slow" beats are found. */


int find_partials(double f1,double f2,int* ndx_limit1,int* ndx_limit2,int upper_window,double stretch)
{
int beat, limit1, limit2;	/* Copies of LIMITS for loop,plus
										"beat" - temporary holding place
										for beat frequency computed. */

double f1_tone, f2_tone;	/* Overtone frequencies during loop */



for( limit1=1, f1_tone=f1 ; limit1 <= *ndx_limit1 ; limit1++)
	{
	for(limit2=1, f2_tone=f2; limit2 <= *ndx_limit2; limit2++)
		{
		beat = (int) fabs(f2_tone*limit2 - f1_tone*limit1);

		if (beat <= upper_window)
				{
				*ndx_limit1 = limit1;
				*ndx_limit2 = limit2;
				return( TRUE );
				}

		}
	}

return( FALSE );
}



/* Lookup table for symbolic tone names. */


char tones[12][3]={"C", "C#", "D", "D#", "E", "F", "F#", "G", "G#",
					"A", "A#", "B"};


/* This function converts a "tone name" into a "tone number" from 00 to 84.
   The lowest tone is C1 (A0..B0 are not supported), the highest tone
   is C7 ("C88" in obsolete notation.) The character string passed to
   this function must start with a letter A to G, #, and the octave
   number. Ex. C#4, G#3, A5, etc.

   The function returns "ERR" on ERROR, else the "tone number" (0-84) is
   returned. */

int string_to_number( tone_name )
char *tone_name;
{
int semitone;
int octave;


for(semitone = 11;semitone >= 0; semitone--)
	{
	if (0 == strncmp( tone_name, tones[semitone], strlen(tones[semitone])))
		break;
	}

if (semitone < 0) return( ERR );	/* Bad compare - couldn't find it */

octave = atoi( &tone_name[ strlen(tone_name) -1 ] );

if (octave == 0) return(ERR);

return(semitone + 12*(octave-1) );
}


/* This routine converts a 'semitone number' into a character string
   representation. */


char *number_to_string( tone_number, string )
int tone_number;
char *string;
{
int semitone;
int octave;

semitone = tone_number % 12;
octave = tone_number / 12  + 1;

sprintf( string , "%s%1d", tones[ semitone ], octave );

return( string );
}




/* Convert a string to upper-case for parsing. */

void ulc(char* str )
{
for(;(*str);str++)
	*str = toupper(*str);
}


/* Compute the beat rate of two specified note values */

void beat_rate_compute()
{
double f1,f2,stretch,pitch;
int limit1,limit2;
int rate_upper;
int result;
int note;
char temp[25];



printf("\nBeat rate calculator\n\n");


printf("A4 is equal to what frequency (440 Hz typical)?");
gets(temp);
pitch = atof( temp );

printf("Enter symbolic value for note 1:");
gets(temp);
ulc(temp); 

note = string_to_number( temp );
if (note == ERR) return;
f1 = find_frequency( note, pitch );

printf("Enter symbolic value for note 2:");
gets(temp);
ulc(temp); 
note = string_to_number( temp );
if (note == ERR) return;
f2 = find_frequency( note, pitch );



printf("Enter overtone number search limit for F1 and F2:");
scanf("%d", &limit1);
limit2=limit1;
printf("Enter beat rate upper limit for search:");
scanf("%d", &rate_upper);

stretch = 1.;

result = find_partials( f1, f2, &limit1, &limit2, rate_upper, stretch);

if (result == FALSE)
	{
	printf("Sorry, no beats less than or equal to %d Hz found in series!", 
			rate_upper);
	return;
	}

printf("The computed beat occurs between %2d*F1 (%lf) and %2d*F2 (%lf),\n",
		limit1, limit1*f1, limit2, limit2*f2 );

printf("The computed beat frequency is %lf Hz\n",
		fabs( limit1*f1 - limit2*f2)	);



}




void main()
{
beat_rate_compute();
}

