/*  LPRINT.C    This is a program for printing C source files.  Syntax:

       LPRINT (filename)

    Tabs are set to 4, pagination at 55 (11 blank lines per page of paper.) */



#include <stdio.h>
#include <string.h>

#define TAB 9
#define FF  12
#define ESC 27




FILE* f1;
FILE* f2;		/* file pointers */
char name[40];	/* filename for input */
int tabsize=4;	/* tab-stop size */

int topmar=4;	/* 4 blanklines at top of each page */
int pagelen=51;	/* page-length for auto-pagination */



/* This function does tab correction.  It takes an input buffer, BUF1,
   and converts it into a useable output buffer, BUF2, with expanded
   tabs.

   Global variable TABSIZE decides how wide to make the tab zones. */


void convert(buf1,buf2)
char *buf1,*buf2;
{
int bp1,bp2;

for(bp2=0;bp2<254;bp2++)	buf2[bp2]=0x20;

buf2[254]=0;

bp1=0;
bp2=0;

do	{
	if (buf1[bp1] != TAB )	{
							buf2[bp2]=buf1[bp1];
							bp2++;
							}	
	else	bp2=tabsize*(bp2/tabsize) + tabsize;

	bp1++;
	}
	while (buf1[bp1-1]);


}




/* Make top margin according to TOPMAR global variable */


void make_top_margin()
{
int i;

for(i=0;i<topmar;i++) fputc( '\n', f1);
}




/* Print a file.  File pointer f1 must point to a valid write-file structure.
   Will use handle f2 for the input file.  [ESC] key halts printing. */


int print_file(filename)
char *filename;
{
char input_buffer[255];
char output_buffer[255];
char* szResult;

int lines;

f2=fopen(filename,"r");

if (f2==0)	{
			printf("\nCouldn't open %s \n\007",filename);
			return(0);
			}

lines=0;
make_top_margin();

do	{
	

	szResult=fgets(input_buffer,253,f2);	/* get a line from user's file */
	convert(input_buffer,output_buffer);	/* fix up the tabs */
	fputs(output_buffer,f1);	/* write to output file */	

/* Do auto-pagination */

	if (++lines==pagelen)	{
							fputc(FF, f1);
							make_top_margin();
							lines=0;
							}
	}
	while (szResult!= NULL);	/* do 'till end of file */

fputc( FF, f1);	/* drop in FORM FEED at EOF */

fclose(f2);
return(0);
}



void main(argc,argv)
int argc;
char *argv[];
{

printf("\nLPRINT C SOURCE PRINTING UTILITY 1.0 V030891");

if (argc < 2)	{
				printf("\n\nFilename to print?");
				scanf("%s", name);
				if (name[0]==0) return;
				}
				else
				{
				strcpy(name, argv[1]);
				}

printf("\n\nPRINTING: %s   (PRESS [ESC] TO HALT)\n\n", name );

f1=fopen("PRN", "w");
print_file(name);
fclose(f1);

}

