/* This is a demonstration of how to do bit-level PORT I/O from
   Borland Turbo C.

   The I/O device is a simple module connected to the parallel printer
   interface. The DB-25 pin assignments for the project are as follows:

   PIN  I/O      CONNECTED TO
   --------------------------
    2    O       LED 1
    3    O       LED 2
    4    O       LED 3
    5    O       LED 4
    6    O       LED 5
    7    O       LED 6
    8    O       5V BEEPER
    9    O       SWITCH_PULLUP_RESISTOR
   10    I       SWITCH
   18-25 X       GROUND


				*/


unsigned int base_addr;	/* global base address for printer port */

#include <stdio.h>
#include <conio.h>
#include <math.h>

/* Exercise LEDs...flashes the LEDS to test if they work. WARNING:
   GLOBAL VARIABLE "BASE_ADDR" MUST HOLD THE CORRECT PORT ADDRESS
   PRIOR TO CALLING THIS ROUTINE! */

void exercise_leds(void)
{
int i,j,speed;

printf("\nExercising the LEDs...\n");

for(i=0,speed=400;i<20;i++)
	{
	for(j=1;j<0x40;j*=2)
		{
		outportb(base_addr, j);
		delay(speed);
		speed-=3;
		}
	if (kbhit()) 	{
			getch();
			break;
			}
	}

outportb(base_addr , 0);	/* All LEDs off */
}

/* Try the beeper...send a short CW message...*/

void try_beeper(void)
{

/* CW message string: 1 = dah, 0 = dit, 2 = letter space, 3 = word space */

static char msg[]={"101021101310102110131002031021111121102000211033"};
int cw_speed = 75;	/* 75 mS = 1 dit */
int i;
char *msg_ptr;

printf("\nTrying out the beeper...\n");

for(i=0;i<4;i++,cw_speed-=20)
  {
  for(msg_ptr=msg; (*msg_ptr) ; msg_ptr++)
	{
	switch(*msg_ptr)
		{
		case '1': outportb( base_addr , 0x40);
		case '3': delay( 3*cw_speed);	/* "dah" */
			  break;

		case '2': delay( cw_speed);  /* char space= 1 dits */
			  break;

		case '0': outportb( base_addr, 0x40);
			  delay( cw_speed);
			  break;

		default: printf("\nDOH! BAD CODE!\n");
		}
	outportb( base_addr , 0);
	delay(cw_speed); /* Inter-element space = 1 dit */
	}
   if (kbhit() != 0) 	{
			getch();
			break;
			}
   }
}

void try_switch(void)
{
int result,i,j;
float freq[36];
float nfreq;
float semi;
static int tune[]={2,3,4,12,4,12,4,12,12,14,15,16,12,14,16,11,14,12,-1};

outportb(base_addr, 0x80);	/* Enable pullup resistor for switch */

semi = 1.059463094;

for(i=0,nfreq=261.625 * (i+1) ;i<3;i++)
	{
	for(j=0;j<12;j++)
		{
		freq[i*12+j] = nfreq;
		nfreq*=semi;
		}
	}

printf("\nPress the switch to see what happens! Press any key to exit...\n");

do {
   ;
   }
   while(inportb(base_addr + 1) & 0x40);	/* wait for initial press */

i=0;

do {
   result = inportb( base_addr + 1) & 0x40;	/* read the switch */

   /* switch pressed... */

	if (result==0)  {
			delay(10);	/* debounce delay */
			sound( (int) freq[tune[i]] );
			do  {
			    ;
			    }
			    while( 0== (inportb( base_addr + 1) & 0x040));
			i++;
			}
   if (tune[i] < 0) break;	/* end indicator reached -- */
   }
   while( kbhit()==0 );

nosound(); /* turn off the pc speaker */
getch(); /* discard character */
}

/* This function looks at machine zero-page RAM to determine the address
   of the printer port. All PC machines store the address at 0040:0008 */

void get_par_addr(void)
{
unsigned int far *base_ptr;

/* The constant "0x00408" is really the 808x address 0040:0008 */
base_ptr = (unsigned int *) 0x00408;
/* Get the two-byte address (INT) from zero page and save it */
base_addr = *base_ptr;
}

void main(void)
{
unsigned int port;

get_par_addr();

if (base_addr == 0)	{
		printf("\nNO parallel port attached!\n");
		exit(1);
		}

printf("\nThe parallel port base address is %4x\n", base_addr);

delay(100);	/* Wait 100 mS */


exercise_leds();
try_beeper();
try_switch();
}
