/*                     PICALC.C


	This is a "dartboard" PI calculator.  It uses the FRAND() function to
"throw" darts at an imaginary corner of a dartboard.  If the FRAND() function
produces a perfectly uniform distribution of points, then it can be
shown that:


        PI   .    Hits Inside Unit Circle
       ----  =   -------------------------
        4          Total Darts Thrown


	Given a unit circle with radius R=1, and  0 <= X <= 1  and 0 <= Y <=1 ,
a dart will be defined as a "hit" if it falls within the circle's radius;
in other words,

      (X^2 + Y^2) ^(1/2)  <  1   means a "hit."		*/



#include <stdio.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>


void main()
{
double result,hits,x,y;
int i,throws;

printf("\n\nWelcome to the dartboard PI calculator.\n\n\
How many darts would you like me to throw?");

scanf("%d",&throws);

printf("\n\nVery well. I'm doing the %d throws now. Hold on...",throws);


//
// Seed the random number generator by using the current
// time
//

srand( (unsigned)time( NULL ) );



/* Do calculation */

hits=0;


//
// Pick values for x and y. NOTE that the values are
// restricted to quadrant I.
//

for(i=0;i<throws;i++)
				{
				//
				// calculate X and Y as random magnitudes.
				// The integer result from RAND() is coerced
				// into a FLOAT having a value between 0 and
				// 1.
				//

				x= (rand() % 32767) / 32767.;
				y= (rand() % 32767) / 32767.;

				if ( sqrt(x*x + y*y) <= 1.)  hits++;
				}


printf("\n\nOK...Out of %d darts thrown, I got %5.0lf hits...\n\n",throws,hits);

result=4*hits/(double) throws;

printf("PI must be about %2.9lf\n\n",result);

}


