
/*                    SORT2.C

	This is an example of a sorted list.  The procedure used is
	a bubble sort.  Instead of swapping items individually, only
	pointers or indices to the items are swapped.  This is a first step
	towards achieving a high-performance sort algorithm.  Swapping
	actual items wastes valuable time, especially when long sorts
	are involved.  */



#include <stdio.h>
#include <string.h>
#include <stdlib.h>



#define MAX 50			/* maximum array row subscript value +1 */
char strings[MAX][80];	/* array of 50 x 80, enough for 50 items */
char pointer[MAX];		/* array of pointers to the STRINGS. Note
							that these aren't really C POINTERS. */
int entries;			/* counter for number of entries */






/* Sort routine.  A shell sort is used as before.  The only difference
   now is that only POINTER values are exchanged, instead of entire
   strings.  Note that these are not "C" pointers; they are just
   array indices. */


void sort_items()
{
int swap_flag,i,j,temp_ptr;

if (entries==0) return;	/* if only one entry, no sort needed ! */


do	{
	swap_flag=0;
	for(i=0,j=1;i<entries;i++,j++)
				{
				if (strcmp(strings[pointer[i] ],
								strings[pointer[j] ]) > 0)
									{
									temp_ptr=pointer[i];
									pointer[i]=pointer[j];	/* ptr swap */
									pointer[j]=temp_ptr;
									swap_flag=1;
									}

				}

	}
	while (swap_flag != 0);
}





/* Main program for sort2 */


void main()
{
int i;

printf("\n\nThis is an example of a shell or bubble sort.  You are to enter\n\
up to 50 items you want sorted.  Press RETURN on a blank line to\n\
begin the sorting procedure.\n\n");

entries=-1;			/* flag for NO entries */

for(i=0;i<MAX;i++)	{
					printf("Item #%3d : ",i+1);
					gets(strings[i]);

					pointer[i]=i;		/* initial value of each pointer. */

					if (strings[i][0] == 0)	break;

					entries++;
					}


/* Check for NO entries condition */

if (entries== -1)	return;	/* blast outta here, nothing to do */

printf("\n\nSorting %d entries...",entries+1);

sort_items();

printf("done.  Here is the sorted list:\n\n");

for(i=0;i<=entries;i++)	
				printf("\nItem #%3d Pointer[%2d] : %s",i+1,
									pointer[i],strings[pointer[i]]);


printf("\n\nThat's all, folks!\n");

}


